@@ -1,3 +1,4 @@
+from cmath import isfinite
 from dataclasses import dataclass
 
 from typing import List, Optional, Tuple, Type, Union, Any, Dict, Callable
@@ -16,10 +17,10 @@ import sys
 
 @dataclass
 class Trainer:
-    config_path: str
+    config_params: Union[Dict, str]
     model: tf.keras.Model
-    train_dataset: Union[Tuple[np.ndarray, np.ndarray], tf.data.Dataset]
-    val_dataset: Union[Tuple[np.ndarray, np.ndarray], tf.data.Dataset] = None
+    train_dataset: Union[Tuple[np.ndarray, np.ndarray],Tuple[np.ndarray, np.ndarray, np.ndarray], tf.data.Dataset]
+    val_dataset: Union[Tuple[np.ndarray, np.ndarray],Tuple[np.ndarray, np.ndarray, np.ndarray], tf.data.Dataset] = None
     strategy: Optional[tf.distribute.Strategy] = None
     random_seed: int = 1234
     validate_yaml: bool = True
@@ -40,16 +41,27 @@ class Trainer:
     _out_path: str = None
     _user_strategy: bool = False
     _trainer_initialized: bool = False
+    _sample_weight_mode: Optional[str] = None
+    
 
     def __post_init__(self):
         """Initialize the trainer: check inputs, load custom modules, parse configuration file."""
 
         if not isinstance(self.model, tf.keras.models.Model):
             raise Exception("model must be keras model")
-        if not isinstance(self.config_path, str) or not os.path.isfile(
-            self.config_path
+        if isinstance(self.train_dataset, tuple):
+            if len(self.train_dataset) <2 or len(self.train_dataset) > 3:
+                raise Exception("train_dataset must be a tuple of (x, y) or (x, y, sample_weight)")
+        if isinstance(self.val_dataset, tuple):
+            if len(self.val_dataset) < 2 or len(self.val_dataset) > 3:
+                raise Exception("val_dataset must be a tuple of (x, y) or (x, y, sample_weight)")
+        if not isinstance(self.config_params, str) and not isinstance(self.config_params, dict):
+            raise Exception("Config_params must be a dictionary or a file path")
+        if isinstance(self.config_params, str) and not os.path.isfile(
+            self.config_params
         ):
-            raise Exception("Config path must be a valid file path")
+            raise Exception("Config_params must be a valid file path")
+
         if self.strategy is not None and not isinstance(
             self.strategy, tf.distribute.Strategy
         ):
@@ -72,6 +84,7 @@ class Trainer:
 
         self._callbacks = self._get_callbacks()
         self._input_shape = self._get_input_shape()
+        self._sample_weight_mode = self._get_sample_weight_mode()
 
         self._trainer_initialized = True
 
@@ -160,6 +173,11 @@ class Trainer:
             print(f"Successfully converted to format {format} ")
 
     
+    def _get_sample_weight_mode(self):
+        """Get the sample weight mode from the config file"""
+        if self.config.sample_weight_mode is not None:
+            return self.config.sample_weight_mode
+
 
     def _get_input_shape(self):
         """Get the input shape of input dataset"""
@@ -250,7 +268,11 @@ class Trainer:
 
     def validate_config(self):
         """Validate again the configuration file. Used after chaning the config parameters"""
-        TrainingModel(**self.config.__dict__)
+
+        temp_dir = {k:v for k,v in self.config.__dict__.items() if v is not None}
+
+        TrainingModel(**temp_dir)
+        self._validate_config_file()
 
 
     @property
@@ -266,7 +288,7 @@ class Trainer:
         self._metrics = self._get_metrics()
 
         self.model.compile(
-            optimizer=self._optimizer, loss=self._loss, metrics=self._get_metrics()
+            optimizer=self._optimizer, loss=self._loss, metrics=self._get_metrics(), sample_weight_mode = self._sample_weight_mode
         )
 
     def _get_strategy(self):
@@ -300,7 +322,11 @@ class Trainer:
         return int(np.argmin(memory_free_values))
 
     def _parse_config(self) -> Any:
-        return yaml_to_pydantic(self.config_path, self.validate_yaml)
+        if isinstance(self.config_params, str):
+            return yaml_to_pydantic(self.config_params, self.validate_yaml)
+        if isinstance(self.config_params, dict):
+            return TrainingModel(**self.config_params)
+
 
     def _validate_config_file(self):
         "Validate existence of the loss, optimizer and callbacks defined in the config file"
@@ -564,9 +590,25 @@ if __name__ == "__main__":
 
     path = "/root/project/yaket/examples/files/trainer.yaml"
 
+    from sklearn.utils.class_weight import compute_sample_weight
+    from sklearn.utils import class_weight
+
+    # define your class weights or automatically compute them
+
+    yargmax = np.argmax(y_train, axis=1)
+    class_weights = class_weight.compute_class_weight('balanced',
+                                                    classes = np.unique(yargmax.flatten()),
+                                                    y = yargmax.flatten())
+    class_weight_dict = dict(enumerate(class_weights))
+
+
+    # compute the sample weights
+    sample_weigth = compute_sample_weight(class_weight_dict, yargmax.flatten()).reshape(yargmax.shape)
+    print(y_train.shape, sample_weigth.shape, x_train.shape)
+
     trainer = Trainer(
-        config_path=path,
-        train_dataset=(x_train, y_train),
+        config_params=path,
+        train_dataset=(x_train, y_train, sample_weigth),
         val_dataset=(x_test, y_test),
         model=model,
         strategy=None,
